@@ -3,6 +3,7 @@ import urllib.parse
 import urllib.error
 import http.client
 import socket
+import gzip
 try :
     import socks
 except ImportError :
@@ -16,27 +17,71 @@ import ulib.tools.pep8 # pylint: disable=W0611
 class ServerError(Exception) :
     def __init__(self, code, text) :
         super(ServerError, self).__init__()
-        self.__code = code
-        self.__text = text
-        self.__message = http.client.responses[code]
+        self._code = code
+        self._text = text
+        self._message = http.client.responses[code]
 
     def code(self) :
-        return self.__code
+        return self._code
 
     def message(self) :
-        return self.__message
+        return self._message
 
     def info(self) :
-        return "%d %s" % (self.__code, self.__message)
+        return "%d %s" % (self._code, self._message)
 
     def text(self) :
-        return self.__text
+        return self._text
 
     def __str__(self) :
-        return "%d %s [%s]" % (self.__code, self.__message, self.__text)
+        return "%d %s [%s]" % (self._code, self._message, self._text)
 
 
 ##### Public classes #####
+class GzipHandler(urllib.request.HTTPHandler) :
+    def __init__(self, debuglevel = 0, only_gzip_flag = False) :
+        self._only_gzip_flag = only_gzip_flag
+        urllib.request.HTTPHandler.__init__(self, debuglevel=debuglevel)
+
+    def http_request(self, request) :
+        request.add_header("Accept-Encoding", "gzip")
+        return request
+
+    def http_response(self, request, response) :
+        if response.headers.get("content-encoding") == "gzip" :
+            gzip_file = gzip.GzipFile(fileobj=response, mode="r")
+            old_response = response
+            response = urllib.response.addinfourl(gzip_file, old_response.headers, old_response.url, old_response.code)
+            response.msg = old_response.msg
+        elif self._only_gzip_flag :
+            print(response.headers)
+            raise RuntimeError("Only gzip!")
+        return response
+
+    https_request = http_request
+    https_response = http_response
+
+
+###
+class NoRedirectsHandler(urllib.request.HTTPRedirectHandler) : # pylint: disable=W0232
+    def http_error_302(self, *args_tuple, **kwargs_dict) :
+        return None
+
+    http_error_301 = http_error_303 = http_error_307 = http_error_302
+
+
+###
+class SocksHandler(urllib.request.HTTPHandler) :
+    def __init__(self, *args_tuple, **kwargs_dict) :
+        self._args_tuple = args_tuple
+        self._kwargs_dict = kwargs_dict
+        urllib.request.HTTPHandler.__init__(self, debuglevel=kwargs_dict.pop("debuglevel", 0))
+
+    def http_open(self, request) :
+        def build(host, port = None, strict = None, timeout = socket._GLOBAL_DEFAULT_TIMEOUT) : # pylint: disable=W0212
+            return SocksConnection(*self._args_tuple, host=host, port=port, strict=strict, timeout=timeout, **self._kwargs_dict)
+        return self.do_open(build, request)
+
 class SocksConnection(http.client.HTTPConnection) :
     def __init__(self, proxy_url = None, proxy_type = None, proxy_host = None, proxy_port = None,
         proxy_user = None, proxy_passwd = None, rdns_flag = True, *args_tuple, **kwargs_dict) :
@@ -58,26 +103,15 @@ class SocksConnection(http.client.HTTPConnection) :
             if proxy_type is None :
                 raise RuntimeError("Invalid SOCKS protocol: %s" % (scheme))
 
-        self.__proxy_args_tuple = (proxy_type, proxy_host, proxy_port, rdns_flag, proxy_user, proxy_passwd)
+        self._proxy_args_tuple = (proxy_type, proxy_host, proxy_port, rdns_flag, proxy_user, proxy_passwd)
 
     def connect(self) :
         self.sock = socks.socksocket()
-        self.sock.setproxy(*self.__proxy_args_tuple)
+        self.sock.setproxy(*self._proxy_args_tuple)
         if not self.timeout is socket._GLOBAL_DEFAULT_TIMEOUT : # pylint: disable=W0212
             self.sock.settimeout(self.timeout)
         self.sock.connect((self.host, self.port))
 
-class SocksHandler(urllib.request.HTTPHandler) :
-    def __init__(self, *args_tuple, **kwargs_dict) :
-        urllib.request.HTTPHandler.__init__(self, debuglevel=kwargs_dict.pop("debuglevel", 0))
-        self.__args_tuple = args_tuple
-        self.__kwargs_dict = kwargs_dict
-
-    def http_open(self, request) :
-        def build(host, port = None, strict = None, timeout = socket._GLOBAL_DEFAULT_TIMEOUT) : # pylint: disable=W0212
-            return SocksConnection(*self.__args_tuple, host=host, port=port, strict=strict, timeout=timeout, **self.__kwargs_dict)
-        return self.do_open(build, request)
-
 
 ##### PEP8 #####
 tools.pep8.setupAliases()
